use tool_helper::Error;
use super::types::Color;

pub enum OutputType {
    TwoIndex,
    OneIndex,
}

pub struct IndexedImage {
    palette:     Vec<Color>,
    data:        Vec<u8>,
    bit_depth:   png::BitDepth,
    output_type: OutputType,
    width:       u16,
    height:      u16,
}

impl IndexedImage {
    pub fn new<R: std::io::Read>(mut reader: png::Reader<R>, output_type: OutputType) -> Result<IndexedImage, Error> {
        let action_name = "Creating IndexedImage";

        let mut buffer = vec![0; reader.output_buffer_size()];
        match reader.next_frame(&mut buffer) {
            Ok(info) => {
                let width     = info.width  as u16;
                let height    = info.height as u16;
                let bit_depth = info.bit_depth;
                
                if info.color_type != png::ColorType::Indexed {
                    return Err(Error::from_str("PNG file must be indexed").with_action(action_name));
                }

                if bit_depth != png::BitDepth::Four && bit_depth != png::BitDepth::Eight {
                    Err(Error::from_str("Only 4 and 8bit color depth are supported").with_action(action_name))
                }

                else {
                    
                    Ok(IndexedImage{palette: Self::make_palette(), data: buffer, bit_depth, output_type, width, height})
                }
            }

            Err(err) => {
                Err(Error::from_error(err).with_action(action_name))
            }
        }
    }

    fn make_palette() -> Vec<Color> {
        Vec::new()
    }
}